/*****
*
* Author     Jean Paquin
* CompuServe [76130,3455]
* Date       July 5th, 1991
* Revision   July 6th, 1991
*
* Syntax     NOACCENT(cString,cOriginal,cSubstitute)
*
* Purpose    Removes accents on vowels and consonants in cString.
*
*            When dealing with foreign names, the problem of the order
*            of appearance in an index inevitably arises. The problem can
*            be solved using a Clipper UDF. Because of speed considerations
*            when indexing huge data files, I decided to program this
*            function in C.
*
*            The lookup and substitution tables can be modified at runtime
*            using optional parameters cOriginal and cSubstitute. You then
*            can remove accents, encode/decode strings, replace undesired
*            characters in a string, etc.
*
* Parameters cString is mandatory
*
*            cOriginal and cSubstitute are optional. Default values are
*            assigned to cOriginal and cSubstitute if not provided by user.
*            If provided, both cOriginal and cSubstitute:
*
*            . must be transmitted simultaneously
*            . must be character strings
*            . must have the same length
*            . length greater than 0 (not empty strings)
*
* CAUTION    1. NOACCENT() works properly with character strings ONLY.
*            2. If an index is built using NOACCENT(), your application
*               will crash if you attempt to update or rebuild the index
*               file and inadvertently forgot to link the function.
*
* Examples   INDEX ON NOACCENT(Last_name + First_Name) TO IDENTITY
*
*            The resulting string is NOT automatically converted to
*            uppercase. The following index expression solves the problem.
*
*            INDEX ON UPPER(NOACCENT(Last_name + First_Name)) TO IDENTITY
*
* Compiling  CL /c /AL /Zl /Od /FPa /Gs proper.c
*
*            /c     compile only, do not automatically invoke the linker
*            /AL    use the large model
*            /Zl    do not place the names of the default librairies
*                   in the object file
*            /Od    defeats optimization so that function will work
*                   properly when linking with Blinker and WarpLink
*            /FPa   do not generate floating point calls
*            /Gs    do not generate stack probes
*
***/

#include "nandef.h"
#include "extend.h"

CLIPPER NOACCENT()
{
   // parameter defaults

   quant  length     = 0;    // if empty or invalid mandatory parameter
    char *string     = "";
                             // if both optional parameters don't meet
                             // conditions described above.

    char *original   = "";
    char *substitute = "CueaaaaceeeiiiAAEooouuyOUaiounNao";

   quant  sublength  = 34;   // length 33 + string terminator '\0'

   // working variables

   quant  i;                 // type (quant) used to process long strings
   quant  j;
     int  found;             // exit while loop when replacement occured

   // mandatory parameter checking

   if (ISCHAR(1) && (quant)_parclen(1) > 0) {

      string = _parc(1);
      length = _parclen(1);

      // optional parameters checking

      if (ISCHAR(2)              &&
          ISCHAR(3)              &&
          (quant)_parclen(2) > 0 &&
          (quant)_parclen(2) == (quant)_parclen(3)) {

          // if optional parameters are character strings
          //    their length is greater than 0 (not empty strings)
          //    are both of same length
          // user parameters supercede default values

            original = _parc(2);
          substitute = _parc(3);
           sublength = (quant)_parclen(2);
      }

      for (i = 0; i < length; i++) {

          found = FALSE;
              j = 0;

          while (j < sublength && !found) {

             if (string[i] == original[j]) {
                 string[i] = substitute[j];
                     found = TRUE;
             }

             j++;

          }
      }
   }

   _retclen(string,length);

}
